import { MEDIA_QUERIES, SPACING_MAP, SPACING_POINTS, WIDTHS } from '../constants';
export function simple(size) {
    const scale = SPACING_POINTS[Math.abs(size)];
    const polarity = size < 0 ? -1 : 1;
    if (scale === undefined) {
        throw Error(`Unknown spacing size ${size} - expected a point from the spacing scale.`);
    }
    return scale * polarity;
}
function styleForDirection(size, property, direction = null) {
    // NB styled-components automatically sets style to include `px` if needed
    return {
        [direction && direction !== 'all' ? `${property}-${direction}` : property]: size,
    };
}
export function responsive({ size, property, direction = null, adjustment = 0, }) {
    const scale = SPACING_MAP[Math.abs(Number(size))];
    const polarity = size < 0 ? -1 : 1;
    if (scale === undefined) {
        throw Error(`Unknown responsive spacing size ${size} - expected a point from the responsive spacing scale.`);
    }
    if (!property) {
        throw Error('No property passed to responsiveSpacing');
    }
    if (Array.isArray(direction)) {
        return Object.assign({}, ...direction.map((dir) => styleForDirection(scale.mobile * polarity + adjustment, property, dir)), {
            [MEDIA_QUERIES.TABLET]: Object.assign({}, ...direction.map((dir) => styleForDirection(scale.tablet * polarity + adjustment, property, dir))),
        });
    }
    const f = Object.assign(Object.assign({}, styleForDirection(scale.mobile * polarity + adjustment, property, direction)), { [MEDIA_QUERIES.TABLET]: styleForDirection(scale.tablet * polarity + adjustment, property, direction) });
    return f;
}
export function responsiveMargin(value) {
    if (Number.isInteger(value)) {
        return responsive({ size: Number(value), property: 'margin' });
    }
    if (typeof value !== 'object') {
        throw Error('Expected padding value to be an object or integer');
    }
    const { size, direction, adjustment } = value;
    return responsive({
        size,
        property: 'margin',
        direction,
        adjustment,
    });
}
export function responsivePadding(value) {
    if (Number.isInteger(value)) {
        return responsive({ size: Number(value), property: 'padding' });
    }
    if (typeof value !== 'object') {
        throw Error('Expected padding value to be an object or integer');
    }
    const { size, direction, adjustment } = value;
    return responsive({
        size,
        property: 'padding',
        direction,
        adjustment,
    });
}
export function withWhiteSpace(config = {}) {
    return ({ margin = config.margin, padding = config.padding, mb: marginBottom = config.marginBottom, } = {}) => {
        const styles = [];
        if (margin !== undefined) {
            if (Array.isArray(margin)) {
                styles.push(...margin.map((val) => responsiveMargin(val)));
            }
            else {
                styles.push(responsiveMargin(margin));
            }
        }
        if (padding !== undefined) {
            if (Array.isArray(padding)) {
                styles.push(...padding.map((val) => responsivePadding(val)));
            }
            else {
                styles.push(responsivePadding(padding));
            }
        }
        if (marginBottom !== undefined) {
            styles.push(responsiveMargin({ size: marginBottom, direction: 'bottom' }));
        }
        return styles;
    };
}
export function withWidth(config = {}) {
    return ({ setWidth = config.width } = {}) => {
        if (setWidth) {
            const width = WIDTHS[setWidth] || setWidth;
            const { mediaQuery = MEDIA_QUERIES.TABLET, noDefault } = config;
            return {
                width: noDefault ? undefined : '100%',
                [mediaQuery]: {
                    width,
                },
            };
        }
        return undefined;
    };
}
